__version__      = '0.2.5'
__url__          = 'https://github.com/innodatalabs/ilabs.s3util'
__author__       = 'Mike Kroutikov'
__author_email__ = 'mkroutikov@innodata.com'
__description__  = 'Utilities to support simple private PyPI on Amazon S3'
__keywords__     = 's3 setup pypi pip'
